<?php
/**
 * DBCart 商城系统
 *
 * ==========================================================================
 * @link      https://www.dbcart.cn/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   https://www.dbcart.cn/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Goods\Repository;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\Query\Expr\Join;
use Goods\Entity\Goods;
use Goods\Entity\GoodsBrand;
use Goods\Entity\GoodsBrandContent;
use Goods\Entity\GoodsClass;
use Goods\Entity\GoodsClassContent;
use Goods\Entity\GoodsContent;
use Goods\Entity\GoodsDiscount;
use Goods\Entity\GoodsGroupPrice;
use Goods\Entity\GoodsImage;
use Goods\Entity\GoodsInClass;
use Goods\Entity\GoodsInTag;
use Goods\Entity\GoodsSalesRecord;
use Goods\Entity\GoodsStock;
use Goods\Repository\QuerySearch\GoodsSearch;
use PluginGroupShow\Entity\GroupShow;

class GoodsRepository extends EntityRepository
{
    /**
     * 获取最大商品id
     * @return int|mixed|string
     * @throws \Doctrine\ORM\NoResultException
     * @throws \Doctrine\ORM\NonUniqueResultException
     */
    public function getMaxGoodsId()
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('MAX(g.goodsId) as maxGoodsId')->from(Goods::class, 'g');

        return $query->getQuery()->getSingleScalarResult();
    }

    /**
     * 后台商品列表返回
     * @param array $search
     * @return \Doctrine\ORM\Query
     */
    public function findAllGoods(array $search = [])
    {
        $query = $this->getEntityManager()->createQueryBuilder()
            ->select('g', 'i.goodsThumbnailImage', 'c.classState')
            ->addSelect("(SELECT cl.className FROM ".GoodsClassContent::class." AS cl WHERE cl.classId=g.classId AND cl.languageCode='".SHOP_FONT_LANGUAGE."') AS className")
            ->addSelect("(SELECT cg.goodsName FROM ".GoodsContent::class." AS cg WHERE cg.goodsId=g.goodsId AND cg.languageCode='".SHOP_FONT_LANGUAGE."') AS goodsName")
            ->from(Goods::class, 'g')
            ->leftJoin(GoodsImage::class, 'i', Join::WITH, 'g.goodsImageId=i.goodsImageId')
            ->leftJoin(GoodsClass::class, 'c', Join::WITH, 'g.classId=c.classId')
            ->orderBy('g.goodsId', 'DESC');

        $query = GoodsSearch::querySearch($search, $query);

        return $query->getQuery();
    }

    /**
     * 品牌商品列表
     * @param $brandId
     * @return \Doctrine\ORM\Query
     */
    public function findBrandGoods($brandId)
    {
        $query = $this->getEntityManager()->createQueryBuilder()
            ->select('g')
            ->addSelect("(SELECT c.goodsName FROM ".GoodsContent::class." AS c WHERE c.goodsId=g.goodsId AND c.languageCode='".SHOP_FONT_LANGUAGE."') AS goodsName")
            ->from(Goods::class, 'g')
            ->where('g.brandId = :brandId')->setParameter('brandId', $brandId)
            ->orderBy('g.goodsId', 'DESC');

        return $query->getQuery();
    }

    /*==============================================下面为前台调用==============================================*/

    /**
     * 前台分类商品列表
     * @param $languageCode
     * @param array $search
     * @param $userGroupId
     * @return \Doctrine\ORM\Query
     */
    public function findShopClassGoodsList($languageCode, array $search = [], $userGroupId = 0)
    {
        $query = $this->getEntityManager()->createQueryBuilder()
            ->select('g', 'gc.goodsName', 'gc.goodsExtendName', 'i.goodsThumbnailImage', 'c.classId', 'r.goodsSalesNum')
            ->from(Goods::class, 'g')
            ->leftJoin(GoodsImage::class, 'i', Join::WITH, 'g.goodsImageId=i.goodsImageId')
            ->innerJoin(GoodsSalesRecord::class, 'r', Join::WITH, 'g.goodsId = r.goodsId')
            ->innerJoin(GoodsInClass::class, 'c', Join::WITH, 'g.goodsId=c.goodsId')
            ->innerJoin(GoodsContent::class, 'gc', Join::WITH, 'g.goodsId=gc.goodsId')
            ->where('g.goodsState = :goodsState')->setParameter('goodsState', 1)
            ->andWhere('gc.languageCode = :languageCode')->setParameter('languageCode', $languageCode)
            ->orderBy('c.classGoodsRecommend', 'ASC')->addOrderBy('c.classGoodsSort', 'ASC');

        //如果会员组存在，则获取会员组价格
        if($userGroupId > 0) $query->addSelect('(SELECT p.userGroupPrice FROM '.GoodsGroupPrice::class.' AS p WHERE p.goodsId=g.goodsId AND p.userGroupId='.$userGroupId.') AS goodsGroupPrice');

        //加入对客户组查看权限的支持
        if (class_exists(GroupShow::class)) {
            $query->innerJoin(GroupShow::class, 's', Join::WITH, 's.goodsId=g.goodsId')
                ->andWhere('s.userGroupId = :userGroupId')->setParameter('userGroupId', intval($userGroupId));
        }

        //排序
        if(isset($search['sort'])) {
            if(in_array($search['sort'], ['priceASC', 'priceDESC'])) $query->addOrderBy('g.goodsPrice', ($search['sort'] == 'priceASC' ? 'ASC' : 'DESC'));
            if(in_array($search['sort'], ['clickASC', 'clickDESC'])) $query->addOrderBy('g.goodsClick', ($search['sort'] == 'clickASC' ? 'ASC' : 'DESC'));
            if(in_array($search['sort'], ['timeASC', 'timeDESC'])) $query->addOrderBy('g.goodsAddTime', ($search['sort'] == 'timeASC' ? 'ASC' : 'DESC'));
        } else $query->addOrderBy('c.goodsId', 'DESC');

        $query = GoodsSearch::shopQuerySearch($search, $query);

        return $query->getQuery();
    }

    /**
     * 前台品牌商品列表
     * @param $languageCode
     * @param array $search
     * @param $userGroupId
     * @return \Doctrine\ORM\Query
     */
    public function findShopBrandGoodsList($languageCode, array $search = [], $userGroupId = 0)
    {
        $query = $this->getEntityManager()->createQueryBuilder()
            ->select(
                'g',
                'gc.goodsName', 'gc.goodsExtendName',
                'i.goodsThumbnailImage',
                'r.goodsSalesNum'
            )
            ->from(Goods::class, 'g')
            ->leftJoin(GoodsImage::class, 'i', Join::WITH, 'g.goodsImageId=i.goodsImageId')
            ->innerJoin(GoodsSalesRecord::class, 'r', Join::WITH, 'g.goodsId = r.goodsId')
            ->innerJoin(GoodsClass::class, 'c', Join::WITH, 'g.classId = c.classId')
            ->innerJoin(GoodsContent::class, 'gc', Join::WITH, 'g.goodsId=gc.goodsId')
            ->where('c.classState = :classState')->setParameter('classState', 1)
            ->andWhere('gc.languageCode = :languageCode')->setParameter('languageCode', $languageCode)
            ->andWhere('g.goodsState = :goodsState')->setParameter('goodsState', 1);

        //如果会员组存在，则获取会员组价格
        if($userGroupId > 0) $query->addSelect('(SELECT p.userGroupPrice FROM '.GoodsGroupPrice::class.' AS p WHERE p.goodsId=g.goodsId AND p.userGroupId='.$userGroupId.') AS goodsGroupPrice');

        //加入对客户组查看权限的支持
        if (class_exists(GroupShow::class)) {
            $query->innerJoin(GroupShow::class, 's', Join::WITH, 's.goodsId=g.goodsId')
                ->andWhere('s.userGroupId = :userGroupId')->setParameter('userGroupId', intval($userGroupId));
        }

        //排序
        if(isset($search['sort'])) {
            if(in_array($search['sort'], ['priceASC', 'priceDESC'])) $query->addOrderBy('g.goodsPrice', ($search['sort'] == 'priceASC' ? 'ASC' : 'DESC'));
            if(in_array($search['sort'], ['clickASC', 'clickDESC'])) $query->addOrderBy('g.goodsClick', ($search['sort'] == 'clickASC' ? 'ASC' : 'DESC'));
            if(in_array($search['sort'], ['timeASC', 'timeDESC'])) $query->addOrderBy('g.goodsAddTime', ($search['sort'] == 'timeASC' ? 'ASC' : 'DESC'));
        } else $query->addOrderBy('g.goodsId', 'DESC');

        $query = GoodsSearch::shopBrandQuerySearch($search, $query);

        return $query->getQuery();
    }

    /**
     * 编辑器商品详情
     * @param $goodsId
     * @param $languageCode
     * @return float|int|mixed|string|null
     * @throws \Doctrine\ORM\NonUniqueResultException
     */
    public function findInsertEditorGoods($goodsId, $languageCode)
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('g', 'i.goodsThumbnailImage', 'i.goodsImage', 'c.goodsName')
            ->from(Goods::class, 'g')
            ->innerJoin(GoodsContent::class, 'c', Join::WITH, 'g.goodsId=c.goodsId')
            ->leftJoin(GoodsImage::class, 'i', Join::WITH, 'g.goodsImageId=i.goodsImageId')
            ->where('g.goodsId = :goodsId')->setParameter('goodsId', $goodsId)
            ->andWhere('c.languageCode = :languageCode')->setParameter('languageCode', $languageCode)
            ->andWhere('g.goodsState = :goodsState')->setParameter('goodsState', 1);

        return $query->getQuery()->getOneOrNullResult();
    }

    /**
     * 前台商品详情
     * @param $goodsId
     * @param $languageCode
     * @param $userGroupId
     * @return float|int|mixed|string|null
     * @throws \Doctrine\ORM\NonUniqueResultException
     */
    public function findShopGoodsInfo($goodsId, $languageCode, $userGroupId = -1)
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select(
            'g',
            'i.goodsThumbnailImage', 'i.goodsImage',
            'r.goodsSalesNum',
            'd.discountPrice', 'd.discountStartTime', 'd.discountEndTime', 'd.discountUserGroupType', 'd.discountUserGroup',
            's.stockNum', 's.stockShowState', 's.stockShow', 's.stockOutSetNum', 's.stockOutShow', 's.cartBuyMinNum', 's.cartBuyMaxNum'
        )
            ->addSelect("(SELECT b.brandName FROM ".GoodsBrandContent::class." as b WHERE b.brandId=g.brandId AND b.languageCode='".$languageCode."') AS brandName")
            ->from(Goods::class, 'g')
            ->leftJoin(GoodsImage::class, 'i', Join::WITH, 'g.goodsImageId=i.goodsImageId')
            ->innerJoin(GoodsSalesRecord::class, 'r', Join::WITH, 'g.goodsId = r.goodsId')
            ->innerJoin(GoodsDiscount::class, 'd', Join::WITH, 'g.goodsId = d.goodsId')
            ->innerJoin(GoodsStock::class, 's', Join::WITH, 'g.goodsId = s.goodsId')
            ->where('g.goodsId = :goodsId')->setParameter('goodsId', $goodsId);

        //如果是预览，则不进行上架判断
        if (isset($_GET['show']) && $_GET['show'] == 'dbshopPreview')  $query->andWhere('g.goodsAddTime = :goodsAddTime')->setParameter('goodsAddTime', isset($_GET['time']) ? intval($_GET['time']) : '');
        else $query->andWhere('g.goodsState = :goodsState')->setParameter('goodsState', 1);

        //加入对客户组查看权限的支持
        if (class_exists(GroupShow::class) && $userGroupId >= 0) {
            $query->innerJoin(GroupShow::class, 'ss', Join::WITH, 'ss.goodsId=g.goodsId')
                ->andWhere('ss.userGroupId = :userGroupId')->setParameter('userGroupId', intval($userGroupId));
        }

        return $query->getQuery()->getOneOrNullResult();
    }

    /**
     * 前台商品搜索
     * @param $languageCode
     * @param array $search
     * @param $userGroupId
     * @return \Doctrine\ORM\Query
     */
    public function findShopSearchGoodsList($languageCode, array $search = [], $userGroupId = 0)
    {
        $query = $this->getEntityManager()->createQueryBuilder()
            ->select('g', 'gc.goodsName', 'gc.goodsExtendName', 'i.goodsThumbnailImage', 'r.goodsSalesNum')
            ->from(Goods::class, 'g')
            ->leftJoin(GoodsImage::class, 'i', Join::WITH, 'g.goodsImageId=i.goodsImageId')
            ->innerJoin(GoodsSalesRecord::class, 'r', Join::WITH, 'g.goodsId = r.goodsId')
            ->innerJoin(GoodsClass::class, 'c', Join::WITH, 'g.classId = c.classId')
            ->innerJoin(GoodsContent::class, 'gc', Join::WITH, 'g.goodsId=gc.goodsId')
            ->where('c.classState = :classState')->setParameter('classState', 1)
            ->andWhere('gc.languageCode = :languageCode')->setParameter('languageCode', $languageCode)
            ->andWhere('g.goodsState = :goodsState')->setParameter('goodsState', 1);

        //如果会员组存在，则获取会员组价格
        if($userGroupId > 0) $query->addSelect('(SELECT p.userGroupPrice FROM '.GoodsGroupPrice::class.' AS p WHERE p.goodsId=g.goodsId AND p.userGroupId='.$userGroupId.') AS goodsGroupPrice');

        //加入对客户组查看权限的支持
        if (class_exists(GroupShow::class)) {
            $query->innerJoin(GroupShow::class, 's', Join::WITH, 's.goodsId=g.goodsId')
                ->andWhere('s.userGroupId = :userGroupId')->setParameter('userGroupId', intval($userGroupId));
        }

        //排序
        if(isset($search['sort'])) {
            if(in_array($search['sort'], ['priceASC', 'priceDESC'])) $query->addOrderBy('g.goodsPrice', ($search['sort'] == 'priceASC' ? 'ASC' : 'DESC'));
            if(in_array($search['sort'], ['clickASC', 'clickDESC'])) $query->addOrderBy('g.goodsClick', ($search['sort'] == 'clickASC' ? 'ASC' : 'DESC'));
            if(in_array($search['sort'], ['timeASC', 'timeDESC'])) $query->addOrderBy('g.goodsAddTime', ($search['sort'] == 'timeASC' ? 'ASC' : 'DESC'));
        } else $query->addOrderBy('g.goodsId', 'DESC');

        $query = GoodsSearch::shopSearchGoods($search, $query);

        return $query->getQuery();
    }

    /**
     * 随机商品，当商品数量比较小是，随机的数量不固定
     * @param $languageCode
     * @param int $goodsNum
     * @param int $goodsId
     * @param int $userGroupId
     * @return float|int|mixed|string
     * @throws \Doctrine\ORM\NoResultException
     * @throws \Doctrine\ORM\NonUniqueResultException
     */
    public function findShopRandGoods($languageCode, int $goodsNum, int $goodsId = 0, int $userGroupId = 0)
    {
        $randGoodsId= floor(lcg_value() * $this->getMaxGoodsId());

        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('g', 'gc.goodsName', 'gc.goodsExtendName', 'i.goodsThumbnailImage')
            ->from(Goods::class, 'g')->leftJoin(GoodsImage::class, 'i', Join::WITH, 'g.goodsImageId=i.goodsImageId')
            ->innerJoin(GoodsContent::class, 'gc', Join::WITH, 'g.goodsId=gc.goodsId')
            ->where('g.goodsState = :goodsState')->setParameter('goodsState', 1)
            ->andWhere('gc.languageCode = :languageCode')->setParameter('languageCode', $languageCode)
            ->andWhere('g.goodsId >= :randGoodsId')->setParameter('randGoodsId', $randGoodsId)->setMaxResults($goodsNum);

        if ($goodsId > 0) $query->andWhere('g.goodsId!= :goodsId')->setParameter('goodsId', $goodsId);

        //加入对客户组查看权限的支持
        if (class_exists(GroupShow::class)) {
            $query->innerJoin(GroupShow::class, 's', Join::WITH, 's.goodsId=g.goodsId')
                ->andWhere('s.userGroupId = :userGroupId')->setParameter('userGroupId', intval($userGroupId));
        }

        //如果会员组存在，则获取会员组价格
        if($userGroupId > 0) $query->addSelect('(SELECT p.userGroupPrice FROM '.GoodsGroupPrice::class.' AS p WHERE p.goodsId=g.goodsId AND p.userGroupId='.$userGroupId.') AS goodsGroupPrice');

        return $query->getQuery()->getResult();
    }

    /**
     * 查找数组商品id信息
     * @param array $goodsIdArray
     * @return int|mixed|string
     */
    public function findShopInGoods(array $goodsIdArray)
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('g')->from(Goods::class, 'g')
            ->where('g.goodsState = :goodsState')->setParameter('goodsState', 1)
            ->andWhere($query->expr()->in('g.goodsId', $goodsIdArray));

        return $query->getQuery()->getResult();
    }
}